module AD2DA_FIR_NCO (
	// PLL ports
	rst_b,
	clk_osc4p,	// 80MHz from OSC to PLL6.
	DA1clk,		// 7.5MHz to DA1
	DA2clk,		// 7.5MHz to DA2

	// D/A data
	outDA1,
	outDA2,
	// PPI interface
	ppi_clk,
	ppi_data,
	frame,
	line
	// Monitoring
	);
// Clock management
input		rst_b;			// active low reset
// 80MHz clock from OSC.
input		clk_osc4p;
output		DA1clk,DA2clk;	// 120MHz clock outputs to A/D and D/A
// DA interface
output	[13:0]	outDA1;		// unsigned binary
output	[13:0]	outDA2;		// unsigned binary
// PPI interface
output		ppi_clk;
output	[15:0]	ppi_data;
output		frame,line;
// Monitoring

 wire		rst_h,p6clk;
 reg	[11:0]	 Icount,ns_Icount;
 reg	[11:0]	 Qcount,ns_Qcount;
 //unsigned
 wire	[11:0]	outDA1_12,outDA2_12;
 reg	[13:0]	outDA1,outDA2;
 reg		ns_flag,flag;
 reg	[15:0]	ppi_data0, ns_ppi_data0;
 reg	[15:0]	ppi_data1;

 wire		done1,done2;
 wire		clk15MHz,clk30MHz;
 wire		ppi_clk;
 assign	ppi_clk=clk30MHz;

 assign	rst_h=~rst_b;

// PLL's
 myPLL6	inst_myPLL6 (
	 	     .inclk0(clk_osc4p),
		     .pllena(1'b1),
		     .areset(rst_h),
		     .c0(p6clk),	// 7.5MHz
		     .c1(clk15MHz),
		     .c2(clk30MHz),
		     .e0(DA1clk),	// 7.5MHz
		     .e1(DA2clk));	// 7.5MHz
 // Counter 
 always @(Icount or Qcount)
 begin
	 ns_Icount=Icount+12'h001;
	 ns_Qcount=Qcount-12'h001;
 end

 // FIFO for I-ch Q-ch
 always @(posedge p6clk or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 outDA1<=14'h0000;
		 outDA2<=14'h0000;
		 Icount<=12'h000;
		 Qcount<=12'hfff;
	 end
	 else
	 begin
		 outDA1<={Icount,2'b00};
		 outDA2<={Qcount,2'b00};
		 Icount<=ns_Icount;
		 Qcount<=ns_Qcount;
	 end
 end
 
 // clk15MHz is 15MHz
 always @(negedge clk15MHz or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 flag<=1'b0;
		 ppi_data0<=16'h0000;
	 end
	 else
	 begin
		 flag<=ns_flag;
		 ppi_data0<=ns_ppi_data0;
	 end
 end
 always @(flag or Icount or Qcount or ppi_data0)
 begin
	 ns_flag=flag+1'b1;
	 ns_ppi_data0=ppi_data0;
	 if (flag)
	 begin
		 ns_ppi_data0={Icount,4'h0};
	 end
	 else
	 begin
		 ns_ppi_data0={Qcount,4'h0};
	 end
 end

 // transfer negedge data to posedge data
 always @(posedge clk15MHz or negedge rst_b)
 begin
	 if (~rst_b)
	 begin
		 ppi_data1<=16'h0000;
	 end
	 else
	 begin
		 ppi_data1<=ppi_data0;
	 end
 end
 
FIFO_FIFOcontrol_FSM inst_FIFO_FIFOcontrol_FSM (
						// input
						.clk15MHz(clk15MHz),
						.clk30MHz(clk30MHz),
						.rst_b(rst_b),
				
						.in_data(ppi_data1),	// 16bit data
						// output
						.frame(frame),
						.line(line),
						.ppi_data(ppi_data)	// ppi_data
						);
endmodule
