module C271 (
		// input
		clk,		// must be 30MHz
		cnt_en,
		rst_b,
		// output
		frame,
		line,
		tx_done
		);
// input
input		clk;		// must be 30MHz
input		cnt_en;
input		rst_b;
		// output
output		frame;
output		line;
output		tx_done;

reg	[8:0]	fcounter, ns_fcounter; // 1-271
reg		frame, ns_frame;
reg	[4:0]	lcounter, ns_lcounter;	// 1-16
reg		line_int, ns_line_int;
wire		line;
assign line=(line_int&frame);
reg		tx_done;
wire		ns_tx_done;
assign	ns_tx_done=(ns_frame==1'b0)&(frame==1'b1);

// frame counter
always @(fcounter or cnt_en)
begin
	ns_fcounter=fcounter;
	if (cnt_en==1)
	begin
		if (fcounter<=272)
			ns_fcounter=fcounter+9'h001;
	end
	else
		ns_fcounter=9'h000;

end

// frame generation
always @(fcounter or frame)
begin
	ns_frame=frame;
	if (fcounter>=1)
	begin
		if (fcounter<=271)
			ns_frame=1'b1;
		else
			ns_frame=1'b0;
	end
end

// line counter
always @(lcounter or cnt_en or frame)
begin
	ns_lcounter=lcounter;
	if (cnt_en==1)
	begin
		if (lcounter==5'h10)
			ns_lcounter=5'h00;
		else
			ns_lcounter=lcounter+5'h01;
	end
	else
		ns_lcounter=5'h00;
end

// line generation
always @(lcounter or line_int)
begin
	if ((lcounter>5'h00)&&(lcounter<=5'h10))
		ns_line_int=1'b1;
	else
		ns_line_int=1'b0;
end

always @(posedge clk or negedge rst_b)
begin
	if (~rst_b)
	begin
		fcounter<=9'h000;
		lcounter<=5'h00;
		frame<=1'b0;
		line_int<=1'b0;
		tx_done<=1'b0;
	end
	else
	begin
		fcounter<=ns_fcounter;
		lcounter<=ns_lcounter;
		frame<=ns_frame;
		line_int<=ns_line_int;
		tx_done<=ns_tx_done;
	end
end

endmodule
