module ManualLoopback (
	// PLL ports
	rst_b,		//SW0
	clk_osc4p,	// 80MHz from OSC to PLL6.
	//DA1clk,		// 7.5MHz to DA1
	//DA2clk,		// 7.5MHz to DA2

	// D/A data
	//outDA1,
	//outDA2,
	//q_out,
	//a_out,
	//frame_out,line_out,
	//ppi_data_out,
	//cs_fsm,
	//frame_cnt,
	//temp,
	//frame_chk,
	// PPI interface
	ppi_clk,
	ppi_data,	// 16-bit bidirection
	frame,		// bidirection
	line,		// bidirection
	// direction control
	// 0-> PPI pins are output,1->PPI pins are input
	oen_b		// SW1 (pressed-->0)
	);

//--------Begin: Port declaration ------------------------------
// PLL ports
input	rst_b;
input	clk_osc4p;	// 80MHz from OSC to PLL6.
//output	DA1clk;		// 7.5MHz to DA1
//output	DA2clk;		// 7.5MHz to DA2

	// D/A data
//output	outDA1;
//output	outDA2;
//output	[15:0]	q_out;
//output	[8:0]	a_out;
//output		frame_out,line_out;
//output	[15:0]	ppi_data_out;
//output	[2:0]	cs_fsm;
//output	[1:0]	frame_cnt;
//output	[15:0]	temp;
//output		frame_chk;
	// PPI interface
output		ppi_clk;
inout	[15:0]	ppi_data;	// 16-bit bidirection
inout		frame;		// bidirection
inout		line;		// bidirection
// direction control
// 0-> PPI pins are output,1->PPI pins are input
input		oen_b;		// SW0 (pressed-->0)
//--------End: Port declaration ------------------------------
//--------Begin: Internal reg/wire declaration ---------------
parameter	IDLE=3'b000, RX_MODE=3'b001, DUMMY1_MODE=3'b010, DUMMY2_MODE=3'b011;
parameter	DUMMY3_MODE=3'b100, DUMMY4_MODE=3'b101, DUMMY5_MODE=3'b110, TX_MODE=3'b111;
reg	[8:0]	address,ns_address;
reg		rst_addr,ns_rst_addr;
reg	[2:0]	cs_fsm,ns_cs_fsm;
reg	[1:0]	frame_cnt,ns_frame_cnt;
reg		tx_en,ns_tx_en,rx_en,ns_rx_en;
reg	[15:0]	ppi_data_out;
wire		tx_done;
wire	[15:0]	temp;
reg		frame_out,frame_out0,frame_out1;
reg		line_out,line_out0,line_out1;
wire		ns_frame_out,ns_line_out;
wire	[15:0]	ns_ppi_data_out;
//--------End: Internal reg/wire declaration ---------------
assign	ppi_data=(~oen_b)? ppi_data_out:16'hzzzz;
assign	frame=(~oen_b)? frame_out:1'bz;
assign	line=(~oen_b)? line_out:1'bz;

//assign	q_out=ppi_data_out;
//assign	a_out=address;
//assign	frame_chk=frame;
assign	ns_ppi_data_out=(~oen_b)? temp:16'h0000;


//------- Begin: Data from Blackfin -----------------------
always @(address or frame or line or rst_addr or ns_frame_out or ns_line_out or oen_b)
begin
	ns_address=address;
	if (rst_addr)
		ns_address=9'h000;
	else
	begin
		if (oen_b)	//RX
		begin
			if (frame&&line)
				ns_address=address+1'b1;
		end
		else 		//TX
		begin
			if (ns_frame_out&&ns_line_out)
				ns_address=address+1'b1;
		end
	end
end

always @(negedge rst_b or posedge ppi_clk)
begin
	if (~rst_b)
	begin
		address<=9'h000;
		ppi_data_out<=16'h0000;
		frame_out0 <= 1'b0;
		frame_out1 <= 1'b0;
		frame_out <= 1'b0;
		line_out0 <= 1'b0;
		line_out1 <= 1'b0;
		line_out <= 1'b0;
	end
	else
	begin
		address<=ns_address;
		ppi_data_out<=ns_ppi_data_out;
		frame_out0 <= ns_frame_out;
		frame_out1 <= frame_out0;
		frame_out <= frame_out1;
		line_out0 <= ns_line_out;
		line_out1 <= line_out0;
		line_out <= line_out1;
	end
end

RAM512	inst_RAM512 (
		    .address(address),
    		    .clock(ppi_clk),
		    .data(ppi_data),
		    .wren(rx_en),
		    .q(temp));
//------- End: Data from Blackfin -----------------------
//------- Begin: Data to Blackfin -----------------------
myPLL	instmyPLL (.inclk0(clk_osc4p),	// 80MHz
		   .c0(ppi_clk));	// 25MHz
C271	instC271  (
		   // input
		   .clk(ppi_clk),
		   .cnt_en(tx_en),
		   .rst_b(rst_b),
		   // output
		   .frame(ns_frame_out),
		   .line(ns_line_out),
		   .tx_done(tx_done));
//------- End: Data from Blackfin -----------------------

//------- FSM for mode change ---------------------------
always @(cs_fsm or rx_en or tx_en or tx_done or rst_addr or frame_cnt or oen_b)
begin
	ns_cs_fsm=cs_fsm;
	ns_rx_en=rx_en;
	ns_tx_en=tx_en;
	ns_rst_addr=rst_addr;
	ns_frame_cnt=frame_cnt;
	case (cs_fsm)
		IDLE:		// rst_b makes address=0
		begin
			if (oen_b)
				ns_cs_fsm=RX_MODE;
		end
		RX_MODE:
		begin
			ns_rx_en=1'b1;
			if (~oen_b)
			begin
				ns_cs_fsm=DUMMY1_MODE;
				ns_rst_addr=1'b1;	// makes address=0 for tx
				ns_frame_cnt=1'b0;
				ns_rx_en=1'b0;
			end
		end
		//-----------------------------------------------------------------
		DUMMY1_MODE:
		begin
			ns_cs_fsm=DUMMY2_MODE;
			ns_rst_addr=1'b1;	// makes address=0 for tx
			ns_frame_cnt=1'b0;
			ns_rx_en=1'b0;
		end
		DUMMY2_MODE:
		begin
			ns_cs_fsm=DUMMY3_MODE;
			ns_rst_addr=1'b1;	// makes address=0 for tx
			ns_frame_cnt=1'b0;
			ns_rx_en=1'b0;
		end
		DUMMY3_MODE:
		begin
			ns_cs_fsm=DUMMY4_MODE;
			ns_rst_addr=1'b1;	// makes address=0 for tx
			ns_frame_cnt=1'b0;
			ns_rx_en=1'b0;
		end
		DUMMY4_MODE:
		begin
			ns_cs_fsm=DUMMY5_MODE;
			ns_rst_addr=1'b1;	// makes address=0 for tx
			ns_frame_cnt=1'b0;
			ns_rx_en=1'b0;
		end
		DUMMY5_MODE:
		begin
			ns_cs_fsm=TX_MODE;
			ns_rst_addr=1'b1;	// makes address=0 for tx
			ns_frame_cnt=1'b0;
			ns_rx_en=1'b0;
		end
		//-----------------------------------------------------------------
		TX_MODE:
		begin
			ns_rst_addr=1'b0;
			ns_tx_en=1'b1;
			if (tx_done)
				ns_frame_cnt=frame_cnt+1'b1;
			if (frame_cnt==2'b11)
			begin
				ns_cs_fsm=IDLE;
				ns_tx_en=1'b0;
			end
		end
	endcase
end

always @(negedge rst_b or posedge ppi_clk)
begin
	if (~rst_b)
	begin
		cs_fsm <=2'b00;
		rx_en <=1'b0;
		tx_en <=1'b0;
		rst_addr <= 1'b0;
		frame_cnt <=1'b0;
	end
	else
	begin
		cs_fsm <= ns_cs_fsm;
		rx_en <= ns_rx_en;
		tx_en <= ns_tx_en;
		rst_addr <= ns_rst_addr;
		frame_cnt <= ns_frame_cnt;
	end
end

endmodule
